@@ -24,6 +24,7 @@
 #include <util/file.h>
 #include <util/log.h>
 #include <diskio/chunkmanager.h>
+#include <diskio/piecedata.h>
 #include <torrent/torrent.h>
 #include <peer/peermanager.h>
 #include <util/error.h>
@@ -33,22 +34,24 @@
 #include <peer/peer.h>
 #include <download/piece.h>
 #include <peer/peerdownloader.h>
+#include <peer/badpeerslist.h>
 #include <util/functions.h>
 #include <interfaces/monitorinterface.h>
 #include <peer/packetwriter.h>
-#include <torrent/ipblocklist.h>
+#include <peer/accessmanager.h>
 #include "chunkselector.h"
 #include "btversion.h"
 #include "webseed.h"
 
+
 namespace bt
 {
-	
-	
 
 	Downloader::Downloader(Torrent & tor,PeerManager & pman,ChunkManager & cman,ChunkSelectorFactoryInterface* fac) 
-	: tor(tor),pman(pman),cman(cman),downloaded(0),tmon(0),chunk_selector(0)
+	: tor(tor),pman(pman),cman(cman),downloaded(0),tmon(0),chunk_selector(0),webseed_endgame_mode(false)
 	{
+		pman.setPieceHandler(this);
+		
 		if (!fac) // check if a custom one was provided, if not create a default one
 			chunk_selector = new ChunkSelector(cman,*this,pman);
 		else
@@ -72,12 +75,27 @@ namespace bt
 				WebSeed* ws = new WebSeed(u,false,tor,cman);
 				webseeds.append(ws);
 				connect(ws,SIGNAL(chunkReady(Chunk*)),this,SLOT(onChunkReady(Chunk*)));
-				connect(ws,SIGNAL(chunkDownloadStarted(ChunkDownloadInterface*)),
-						this,SLOT(chunkDownloadStarted(ChunkDownloadInterface*)));
-				connect(ws,SIGNAL(chunkDownloadFinished(ChunkDownloadInterface*)),
-						this,SLOT(chunkDownloadFinished(ChunkDownloadInterface*)));
+				connect(ws,SIGNAL(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)),
+						this,SLOT(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)));
+				connect(ws,SIGNAL(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)),
+						this,SLOT(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)));
 			}
 		}
+		
+		if (webseeds.count() > 0)
+		{
+			webseed_range_size = tor.getNumChunks() / webseeds.count();
+			if (webseed_range_size == 0)
+				webseed_range_size = 1;
+			
+			// make sure the range is not to big
+			if (webseed_range_size > tor.getNumChunks() / 10)
+				webseed_range_size = tor.getNumChunks() / 10;
+		}
+		else
+		{
+			webseed_range_size = 1;
+		}
 	}
 
 
@@ -111,14 +129,7 @@ namespace bt
 			return;
 		}
 		
-		// if the chunk is not in memory, reload it
-		if (cd->getChunk()->getStatus() == Chunk::ON_DISK)
-		{
-			cman.prepareChunk(cd->getChunk(),true);
-		}
-			
 		bool ok = false;
-		
 		if (cd->piece(p,ok))
 		{
 			if (tmon)
@@ -134,19 +145,22 @@ namespace bt
 					downloaded = 0;
 				else
 					downloaded -= cd->getChunk()->getSize();
+				current_chunks.erase(p.getIndex());
+			}
+			else
+			{
+				current_chunks.erase(p.getIndex());
+				foreach (WebSeed* ws,webseeds)
+				{
+					if (ws->inCurrentRange(p.getIndex()))
+						ws->chunkDownloaded(p.getIndex());
+				}
 			}
-			current_chunks.erase(p.getIndex());
 		}
 		else
 		{
 			if (ok)
 				downloaded += p.getLength();
-			
-			// save to disk again, if it is idle
-			if (cd->isIdle() && cd->getChunk()->getStatus() == Chunk::MMAPPED)
-			{
-				cman.saveChunk(cd->getChunk()->getIndex(),false);
-			}
 		}
 			
 		if (!ok)
@@ -157,6 +171,11 @@ namespace bt
 		}
 	}
 	
+	bool Downloader::endgameMode() const
+	{
+		return current_chunks.count() >= cman.chunksLeft();
+	}
+	
 	void Downloader::update()
 	{
 		if (cman.completed())
@@ -176,7 +195,15 @@ namespace bt
 		
 		foreach (WebSeed* ws,webseeds)
 		{
-			downloaded += ws->update();
+			ws->update();
+		}
+		
+		if (isFinished())
+		{
+			foreach (WebSeed* ws,webseeds)
+			{
+				ws->cancel();
+			}
 		}
 	}
 
@@ -186,22 +213,13 @@ namespace bt
 		for (CurChunkItr j = current_chunks.begin();j != current_chunks.end();++j)
 		{
 			ChunkDownload* cd = j->second;
-			if (cd->isIdle()) // idle chunks do not need to be in memory
+			if (cd->isIdle())
 			{
-				Chunk* c = cd->getChunk();
-				if (c->getStatus() == Chunk::MMAPPED && !webseeds_chunks.find(c->getIndex()))
-				{
-					cman.saveChunk(c->getIndex(),false);
-				}
-			} 
+				continue;
+			}
 			else if (cd->isChoked())
 			{
 				cd->releaseAllPDs();
-				Chunk* c = cd->getChunk();
-				if (c->getStatus() == Chunk::MMAPPED && !webseeds_chunks.find(c->getIndex()))
-				{
-					cman.saveChunk(c->getIndex(),false);
-				}
 			}
 			else if (cd->needsToBeUpdated())
 			{
@@ -211,56 +229,28 @@ namespace bt
 		
 		foreach (PieceDownloader* pd,piece_downloaders)
 		{
-			if (pd->canDownloadChunk())
+			if (!pd->isChoked())
 			{
-				if (!pd->isChoked())
-					downloadFrom(pd);
-				
-				pd->setNearlyDone(false);
+				while (pd->canDownloadChunk())
+				{
+					if (!downloadFrom(pd))
+						break;
+					pd->setNearlyDone(false);
+				}
 			}
 		}
 		
 		foreach (WebSeed* ws,webseeds)
 		{
-			if (!ws->busy())
+			if (!ws->busy() && ws->failedAttempts() < 3)
 			{
 				downloadFrom(ws);
 			}
 		}
 	}
 
-	Uint32 Downloader::maxMemoryUsage()
-	{
-		Uint32 max = 1024 * 1024;
-		switch (mem_usage)
-		{	
-			case 1: // Medium
-				max *= 60; // 60 MB
-				break;
-			case 2: // High
-				max *= 80; // 90 MB
-				break;
-			case 0: // LOW
-			default:
-				max *= 40; // 30 MB
-				break;
-		}
-		return max;
-	}
 	
-	Uint32 Downloader::numNonIdle()
-	{
-		Uint32 num_non_idle = 0;
-		for (CurChunkItr j = current_chunks.begin();j != current_chunks.end();++j)
-		{
-			ChunkDownload* cd = j->second;
-			if (!cd->isIdle())
-				num_non_idle++;
-		}
-		return num_non_idle;
-	}
-	
-	ChunkDownload* Downloader::selectCD(PieceDownloader* pd,Uint32 num)
+	ChunkDownload* Downloader::selectCD(PieceDownloader* pd,Uint32 n)
 	{
 		ChunkDownload* sel = 0;
 		Uint32 sel_left = 0xFFFFFFFF;
@@ -271,7 +261,7 @@ namespace bt
 			if (pd->isChoked() || !pd->hasChunk(cd->getChunk()->getIndex()))
 				continue;
 			
-			if (cd->getNumDownloaders() == num) 
+			if (cd->getNumDownloaders() == n) 
 			{
 				// lets favor the ones which are nearly finished
 				if (!sel || cd->getTotalPieces() - cd->getPiecesDownloaded() < sel_left)
@@ -285,30 +275,15 @@ namespace bt
 	}
 	
 
-	bool Downloader::findDownloadForPD(PieceDownloader* pd,bool warmup)
+	bool Downloader::findDownloadForPD(PieceDownloader* pd)
 	{
 		ChunkDownload* sel = 0;
 		
-		// first see if there are ChunkDownload's which need a PieceDownloader
+		// See if there are ChunkDownload's which need a PieceDownloader
 		sel = selectCD(pd,0);
-		
-		if (!sel && warmup)
-		{
-			// if we couldn't find one, try to select another 
-			// which only has one downloader
-			// so that during warmup, there are at the most 2 downloaders 
-			// assigned to one peer	
-			sel = selectCD(pd,1);
-		}
-		
 		if (sel)
 		{
-			// if it is on disk, reload it
-			if (sel->getChunk()->getStatus() == Chunk::ON_DISK)
-				cman.prepareChunk(sel->getChunk(),true);
-			
-			sel->assign(pd);
-			return true;
+			return sel->assign(pd);
 		}
 		
 		return false;
@@ -334,37 +309,28 @@ namespace bt
 		return cdmin;
 	}
 
-	void Downloader::downloadFrom(PieceDownloader* pd)
+	bool Downloader::downloadFrom(PieceDownloader* pd)
 	{
-		// calculate the max memory usage
-		Uint32 max = maxMemoryUsage();
-		// calculate number of non idle chunks
-		Uint32 num_non_idle = numNonIdle();
-		
 		// first see if we can use an existing dowload
-		if (findDownloadForPD(pd,cman.getNumChunks() - cman.chunksLeft() <= 4))
-			return;
-		
-		bool limit_exceeded = num_non_idle * tor.getChunkSize() >= max;
+		if (findDownloadForPD(pd))
+			return true;
 		
 		Uint32 chunk = 0;
-		if (!limit_exceeded && chunk_selector->select(pd,chunk))
+		if (chunk_selector->select(pd,chunk))
 		{
 			Chunk* c = cman.getChunk(chunk);
 			if (current_chunks.contains(chunk))
 			{
-				if (c->getStatus() == Chunk::ON_DISK)
-					cman.prepareChunk(c,true);
-			
-				current_chunks.find(chunk)->assign(pd);
+				return current_chunks.find(chunk)->assign(pd);
 			}
-			else if (cman.prepareChunk(c))
+			else
 			{
 				ChunkDownload* cd = new ChunkDownload(c);
 				current_chunks.insert(chunk,cd);
 				cd->assign(pd);
 				if (tmon)
 					tmon->downloadStarted(cd);
+				return true;
 			}
 		}
 		else if (pd->getNumGrabbed() == 0)
@@ -374,44 +340,49 @@ namespace bt
 			
 			if (cdmin) 
 			{
-				// if it is on disk, reload it
-				if (cdmin->getChunk()->getStatus() == Chunk::ON_DISK)
-				{
-					cman.prepareChunk(cdmin->getChunk(),true);
-				}
-				
-				cdmin->assign(pd); 
+				return cdmin->assign(pd); 
 			}
 		} 
+		
+		return false;
 	}
 	
 	void Downloader::downloadFrom(WebSeed* ws)
 	{
 		Uint32 first = 0;
 		Uint32 last = 0;
-		if (chunk_selector->selectRange(first,last))
+		webseed_endgame_mode = false;
+		if (chunk_selector->selectRange(first,last,webseed_range_size))
 		{
-			for (Uint32 i = first;i <= last;i++)
-			{
-				cman.prepareChunk(cman.getChunk(i),true);
-				webseeds_chunks.insert(i,ws);
-			}
 			ws->download(first,last);
 		}
+		else
+		{
+			// go to endgame mode
+			webseed_endgame_mode = true;
+			if (chunk_selector->selectRange(first,last,webseed_range_size))
+				ws->download(first,last);
+		}
 	}
 	
 
 	bool Downloader::areWeDownloading(Uint32 chunk) const
 	{
-		return current_chunks.find(chunk) != 0 || webseeds_chunks.find(chunk) != 0;
+		return current_chunks.find(chunk) != 0;
 	}
 	
 	bool Downloader::canDownloadFromWebSeed(Uint32 chunk) const
 	{
-		if (cman.chunksLeft() <= current_chunks.count() + webseeds_chunks.count())
+		if (webseed_endgame_mode)
 			return true;
-		else
-			return !areWeDownloading(chunk);
+		
+		foreach (WebSeed* ws,webseeds)
+		{
+			if (ws->busy() && ws->inCurrentRange(chunk))
+				return false;
+		}
+		
+		return !areWeDownloading(chunk);
 	}
 	
 	Uint32 Downloader::numDownloadersForChunk(Uint32 chunk) const
@@ -426,8 +397,6 @@ namespace bt
 	void Downloader::onNewPeer(Peer* peer)
 	{		
 		PieceDownloader* pd = peer->getPeerDownloader();
-		connect(pd,SIGNAL(downloaded(const bt::Piece& )),
-				this,SLOT(pieceReceived(const bt::Piece& )));
 		piece_downloaders.append(pd);
 	}
 
@@ -449,24 +418,29 @@ namespace bt
 	{
 		Chunk* c = cd->getChunk();
 		// verify the data
-		SHA1Hash h;
-		if (cd->usingContinuousHashing())
-			h = cd->getHash();
-		else
-			h = SHA1Hash::generate(c->getData(),c->getSize());
-
+		SHA1Hash h = cd->getHash();
+		
 		if (tor.verifyHash(h,c->getIndex()))
 		{
 			// hash ok so save it
 			try
 			{
-				cman.saveChunk(c->getIndex());
+				foreach (WebSeed* ws,webseeds)
+				{
+					// tell all webseeds a chunk is downloaded
+					if (ws->inCurrentRange(c->getIndex()))
+						ws->chunkDownloaded(c->getIndex());
+				}
+				
+				cman.chunkDownloaded(c->getIndex());
 				Out(SYS_GEN|LOG_IMPORTANT) << "Chunk " << c->getIndex() << " downloaded " << endl;
 				// tell everybody we have the Chunk
 				for (Uint32 i = 0;i < pman.getNumConnectedPeers();i++)
 				{
 					pman.getPeer(i)->getPacketWriter().sendHave(c->getIndex());
 				}
+				
+				emit chunkDownloaded(c->getIndex());
 			}
 			catch (Error & e)
 			{
@@ -493,10 +467,10 @@ namespace bt
 				Peer* p = pman.findPeer(only);
 				if (!p)
 					return false;
-				QString IP(p->getIPAddresss());
-				Out(SYS_GEN|LOG_NOTICE) << "Peer " << IP << " sent bad data" << endl;
-				IPBlocklist & ipfilter = IPBlocklist::instance();
-				ipfilter.insert( IP );
+				
+				QString ip = p->getIPAddresss();
+				Out(SYS_GEN|LOG_NOTICE) << "Peer " << ip << " sent bad data" << endl;
+				AccessManager::instance().banPeer(ip);
 				p->kill(); 
 			}
 			return false;
@@ -506,20 +480,11 @@ namespace bt
 	
 	void Downloader::clearDownloads()
 	{
-		for (CurChunkItr i = current_chunks.begin();i != current_chunks.end();++i)
-		{
-			Uint32 ch = i->first;
-			Chunk* c = i->second->getChunk();
-			if (c->getStatus() == Chunk::MMAPPED)
-				cman.saveChunk(ch,false);
-			
-			c->setStatus(Chunk::NOT_DOWNLOADED);
-		}
 		current_chunks.clear();
 		piece_downloaders.clear();
 		
 		foreach (WebSeed* ws,webseeds)
-			ws->reset();
+			ws->cancel();
 	}
 	
 	Uint32 Downloader::downloadRate() const
@@ -550,6 +515,14 @@ namespace bt
 			ChunkDownload* cd = i->second;
 			tmon->downloadStarted(cd);
 		}
+		
+		foreach (WebSeed* ws,webseeds)
+		{
+			WebSeedChunkDownload* cd = ws->currentChunkDownload();
+			if (cd)
+				tmon->downloadStarted(cd);
+		}
+		
 	}
 	
 
@@ -618,7 +591,7 @@ namespace bt
 				return;
 			}
 			Chunk* c = cman.getChunk(hdr.index);
-			if (!c->isExcluded() && cman.prepareChunk(c))
+			if (!c->isExcluded())
 			{
 				ChunkDownload* cd = new ChunkDownload(c);
 				bool ret = false;
@@ -631,7 +604,7 @@ namespace bt
 					ret = false;
 				}
 				
-				if (!ret)
+				if (!ret || c->getStatus() == Chunk::ON_DISK)
 				{
 					delete cd;
 				}
@@ -678,23 +651,11 @@ namespace bt
 			if (!c)
 				return num_bytes;
 			
-			Uint32 last_size = c->getSize() % MAX_PIECE_LEN;
-			if (last_size == 0)
-				last_size = MAX_PIECE_LEN;
+			ChunkDownload tmp(c);
+			if (!tmp.load(fptr,hdr,false))
+				return num_bytes;
 			
-			// create the bitset and read it 
-			BitSet bs(hdr.num_bits);
-			fptr.read(bs.getData(),bs.getNumBytes());
-			
-			for (Uint32 j = 0;j < hdr.num_bits;j++)
-			{
-				if (bs.get(j))
-					num_bytes += j == hdr.num_bits - 1 ? 
-							last_size : MAX_PIECE_LEN;
-			}
-			
-			if (hdr.buffered)
-				fptr.seek(File::CURRENT,c->getSize());
+			num_bytes += tmp.bytesDownloaded();
 		}
 		curr_chunks_downloaded = num_bytes;
 		return num_bytes;
@@ -710,8 +671,7 @@ namespace bt
 		for (Uint32 i = from;i <= to;i++)
 		{
 			ChunkDownload* cd = current_chunks.find(i);
-			// finish only seed chunks
-			if (!cd || cman.getChunk(i)->getPriority() == bt::ONLY_SEED_PRIORITY)
+			if (!cd)
 				continue;
 			
 			cd->cancelAll();
@@ -738,13 +698,6 @@ namespace bt
 		chunk_selector->reinsert(chunk);
 	}
 	
-	Uint32 Downloader::mem_usage = 0;
-	
-	void Downloader::setMemoryUsage(Uint32 m)
-	{
-		mem_usage = m;
-	}
-	
 	void Downloader::dataChecked(const BitSet & ok_chunks)
 	{
 		for (Uint32 i = 0;i < ok_chunks.getNumBits();i++)
@@ -771,14 +724,36 @@ namespace bt
 	
 	void Downloader::onChunkReady(Chunk* c)
 	{
-		SHA1Hash h = SHA1Hash::generate(c->getData(),c->getSize());
-		webseeds_chunks.erase(c->getIndex());
+		PieceData* piece = c->getPiece(0,c->getSize(),false);
 		
+		webseeds_chunks.erase(c->getIndex());
+		if (!piece)
+		{
+			// reset chunk but only when no other peer is downloading it
+			if (!current_chunks.find(c->getIndex()))
+				cman.resetChunk(c->getIndex());
+			
+			chunk_selector->reinsert(c->getIndex());
+			return;
+		}
+		
+
+		SHA1Hash h = SHA1Hash::generate(piece->data(),c->getSize());
+		piece->unref();
 		if (tor.verifyHash(h,c->getIndex()))
 		{
 			// hash ok so save it
 			try
 			{
+				downloaded += c->getSize();
+				
+				foreach (WebSeed* ws,webseeds)
+				{
+					// tell all webseeds a chunk is downloaded
+					if (ws->inCurrentRange(c->getIndex()))
+						ws->chunkDownloaded(c->getIndex());
+				}
+				
 				ChunkDownload* cd = current_chunks.find(c->getIndex());
 				if (cd)
 				{
@@ -787,7 +762,8 @@ namespace bt
 					current_chunks.erase(c->getIndex());
 				}
 				
-				cman.saveChunk(c->getIndex());
+				c->savePiece(piece);
+				cman.chunkDownloaded(c->getIndex());
 			
 				Out(SYS_GEN|LOG_IMPORTANT) << "Chunk " << c->getIndex() << " downloaded via webseed ! " << endl;
 				// tell everybody we have the Chunk
@@ -816,15 +792,17 @@ namespace bt
 		}
 	}
 	
-	void Downloader::chunkDownloadStarted(ChunkDownloadInterface* cd)
+	void Downloader::chunkDownloadStarted(WebSeedChunkDownload* cd,Uint32 chunk)
 	{
+		webseeds_chunks.insert(chunk,cd->ws);
 		active_webseed_downloads++;
 		if (tmon)
 			tmon->downloadStarted(cd);
 	}
 	
-	void Downloader::chunkDownloadFinished(ChunkDownloadInterface* cd)
+	void Downloader::chunkDownloadFinished(WebSeedChunkDownload* cd,Uint32 chunk)
 	{
+		webseeds_chunks.erase(chunk);
 		if (active_webseed_downloads > 0)
 			active_webseed_downloads--;
 		
@@ -844,10 +822,10 @@ namespace bt
 		WebSeed* ws = new WebSeed(url,true,tor,cman);
 		webseeds.append(ws);
 		connect(ws,SIGNAL(chunkReady(Chunk*)),this,SLOT(onChunkReady(Chunk*)));
-		connect(ws,SIGNAL(chunkDownloadStarted(ChunkDownloadInterface*)),
-				this,SLOT(chunkDownloadStarted(ChunkDownloadInterface*)));
-		connect(ws,SIGNAL(chunkDownloadFinished(ChunkDownloadInterface*)),
-				this,SLOT(chunkDownloadFinished(ChunkDownloadInterface*)));
+		connect(ws,SIGNAL(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)),
+				this,SLOT(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)));
+		connect(ws,SIGNAL(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)),
+				this,SLOT(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)));
 		return ws;
 	}
 		
@@ -908,10 +886,10 @@ namespace bt
 				WebSeed* ws = new WebSeed(url,true,tor,cman);
 				webseeds.append(ws);
 				connect(ws,SIGNAL(chunkReady(Chunk*)),this,SLOT(onChunkReady(Chunk*)));
-				connect(ws,SIGNAL(chunkDownloadStarted(ChunkDownloadInterface*)),
-						this,SLOT(chunkDownloadStarted(ChunkDownloadInterface*)));
-				connect(ws,SIGNAL(chunkDownloadFinished(ChunkDownloadInterface*)),
-						this,SLOT(chunkDownloadFinished(ChunkDownloadInterface*)));
+				connect(ws,SIGNAL(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)),
+						this,SLOT(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)));
+				connect(ws,SIGNAL(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)),
+						this,SLOT(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)));
 			}
 		}
 	}
@@ -923,6 +901,11 @@ namespace bt
 			ws->setGroupIDs(up,down);
 		}
 	}
+	
+	ChunkDownload* Downloader::getDownload(Uint32 chunk)
+	{
+		return current_chunks.find(chunk);
+	}
 }
 
 #include "downloader.moc"
