@@ -48,14 +48,12 @@ class PropertyContainer;
 
 /**
  * The Kopete::Account class handles one account.
- * Each protocol should subclass this class in its own custom accounts class.
- * There are few pure virtual method that the protocol must implement. Examples are:
+ * Each protocol implementation should subclass this class in its own custom account class.
+ * There are a few pure virtual methods that must be implemented. Examples are:
  * \li \ref connect()
  * \li \ref disconnect()
  * \li \ref createContact()
  *
- * If your account requires a password, derive from @ref PasswordedAccount instead of this class.
- *
  * The accountId is an @em constant unique id, which represents the login.
  * The @ref myself() contact is one of the most important contacts, which represents
  * the user tied to this account. You must create this contact in the contructor of your
@@ -123,12 +121,18 @@ public:
 	Q_DECLARE_FLAGS(OnlineStatusOptions, OnlineStatusOption)
 
 	/**
+	 * Constructor for the Account object.
+	 * 
 	 * @param parent the protocol for this account. The account is a child object of the
 	 * protocol, so it will be automatically deleted when the protocol is.
 	 * @param accountID the unique ID of this account.
 	 * @param name the name of this QObject.
 	 */
 	Account(Protocol *parent, const QString &accountID);
+
+	/**
+	 * Destroy the Account object.
+	 */
 	~Account();
 
 	/**
@@ -142,7 +146,10 @@ public:
 	QString accountId() const;
 
 	/**
-	 * \return The label of this account, for the GUI
+	 * The label for this account.
+	 *
+	 * This is used in the GUI.
+	 * \return The label of this account
 	 */
 	QString accountLabel() const;
 
@@ -278,7 +285,7 @@ public:
 	virtual bool hasCustomStatusMenu() const;
 
 	/**
-	 * @brief Retrieve the list of contacts for this account
+	 * @brief Retrieve the list of contacts for this account (except myself contact)
 	 *
 	 * The list is guaranteed to contain only contacts for this account,
 	 * so you can safely use static_cast to your own derived contact class
@@ -490,7 +497,7 @@ public:
 	 * Register a new Contact with the account. This should be called @em only from the
 	 * @ref Contact constructor, not from anywhere else (not even a derived class).
 	 */
-	void registerContact( Contact *c );
+	bool registerContact( Contact *c );
 
 public slots:
 	/**
