@@ -203,11 +203,18 @@ private slots:
 	 */
 	void unregisterAccount( const Kopete::Account *account );
 
+	void removeAccountConnectedChanged();
+
+	/**
+	 * Remove account and its contacts.
+	 */
+	void removeAccountInternal();
+
 private:
 	bool isAnyAccountConnected() const;
 	static AccountManager *s_self;
 	class Private;
-	Private *d;
+	Private * const d;
 };
 
 } //END namespace Kopete
