@@ -38,18 +38,20 @@
 #include "bonjourcontactconnection.h"
 
 
+static const char AvailabilityStatusAvailId[] = "avail";
+static const char AvailabilityStatusAwayId[] =  "away";
+// TODO: add this status to the account
+// static const char AvailabilityStatusDnDId[] =   "dnd";
+
+
 BonjourAccount::BonjourAccount( BonjourProtocol *parent, const QString& accountID )
-: Kopete::Account ( parent, accountID )
+: Kopete::Account ( parent, accountID ), username(), firstName(), emailAddress(), lastName(),
+	service(NULL), localServer(NULL), listeningPort(0), bonjourGroup(NULL), browser(NULL), unknownConnections()
 {
 	// Init the myself contact
 	setMyself( new BonjourContact( this, accountId(), accountId(), Kopete::ContactList::self()->myself() ) );
 	myself()->setOnlineStatus( BonjourProtocol::protocol()->bonjourOffline );
 
-	service = NULL;
-	localServer = NULL;
-	listeningPort = 0;
-	browser = NULL;
-
 	// All Contacts Go To The Bonjour Group
 	bonjourGroup = Kopete::ContactList::self()->findGroup("Bonjour");
 
@@ -61,10 +63,10 @@ BonjourAccount::BonjourAccount( BonjourProtocol *parent, const QString& accountI
 
 void BonjourAccount::parseConfig()
 {
-	username = configGroup()->readEntry("username").toUtf8();
-	firstName = configGroup()->readEntry("firstName").toUtf8();
-	lastName = configGroup()->readEntry("lastName").toUtf8();
-	emailAddress = configGroup()->readEntry("emailAddress").toUtf8();
+	username = configGroup()->readEntry("username").toLatin1();
+	firstName = configGroup()->readEntry("firstName").toLatin1();
+	lastName = configGroup()->readEntry("lastName").toLatin1();
+	emailAddress = configGroup()->readEntry("emailAddress").toLatin1();
 }
 
 BonjourAccount::~BonjourAccount()
@@ -169,7 +171,7 @@ void BonjourAccount::startPublish()
         map.insert("last", lastName);
         map.insert("node", "kopete");
         map.insert("port.p2pj", QByteArray::number(listeningPort));	// This Number Actually Ignored
-        map.insert("status", "avail");
+        map.insert("status", AvailabilityStatusAvailId);
         map.insert("txtvers", "1");
         map.insert("vc", "!");
         map.insert("ver", "0.0.1");
@@ -190,7 +192,7 @@ void BonjourAccount::published(bool success)
 		kDebug()<<"Publish Failed";
 		disconnect();
 		KMessageBox::queuedMessageBox(Kopete::UI::Global::mainWidget(), KMessageBox::Error, 
-		i18n("Unable to publish Bonjour service. Currently the Bonjour plugin only works with Avahi"));
+		i18n("Unable to publish Bonjour service. Currently the Bonjour plugin only works with Avahi."));
 	}
 }
 
@@ -201,7 +203,7 @@ void BonjourAccount::connect( const Kopete::OnlineStatus& /* initialStatus */ )
 
 	if (DNSSD::ServiceBrowser::isAvailable() != DNSSD::ServiceBrowser::Working) {
 		KMessageBox::queuedMessageBox(Kopete::UI::Global::mainWidget(), KMessageBox::Error, 
-		i18n("Sorry, we are unable to connect to the local mDNS server. Please ensure the Avahi daemon is running."));
+		i18n("Unable to connect to the local mDNS server. Please ensure the Avahi daemon is running."));
 		return;
 	}
 
@@ -270,7 +272,7 @@ void BonjourAccount::goingOffline(DNSSD::RemoteService::Ptr pointer)
 	pointer->resolve();
 
 	// In case we have lost connection, this may return NULL
-	Kopete::Contact *c = contacts()[pointer->serviceName()];
+	Kopete::Contact *c = contacts().value( pointer->serviceName() );
 
 	if (c)
 		c->setOnlineStatus(Kopete::OnlineStatus::Offline);
@@ -333,8 +335,14 @@ void BonjourAccount::slotGoOnline ()
 
 	if (!isConnected())
 		connect();
-	else
+	else {
+		if (service) {
+			QMap <QString, QByteArray> map = service->textData();
+			map["status"] = AvailabilityStatusAvailId;
+			service->setTextData(map);
+		}
 		myself()->setOnlineStatus( BonjourProtocol::protocol()->bonjourOnline );
+	}
 }
 
 void BonjourAccount::slotGoAway ()
@@ -344,6 +352,12 @@ void BonjourAccount::slotGoAway ()
 	if (!isConnected ())
 		connect();
 
+	if (service) {
+		QMap <QString, QByteArray> map = service->textData();
+		map["status"] = AvailabilityStatusAwayId; // "dnd" would be another option here
+		service->setTextData(map);
+	}
+
 	myself()->setOnlineStatus( BonjourProtocol::protocol()->bonjourAway );
 }
 
@@ -363,7 +377,7 @@ void BonjourAccount::receivedMessage( const QString &message )
 	BonjourContact* messageSender;
 
 	from = message.section( ':', 0, 0 );
-	Kopete::Contact* contact = contacts()[from];
+	Kopete::Contact* contact = contacts().value(from);
 	messageSender = dynamic_cast<BonjourContact *>( contact );
 }
 
@@ -477,10 +491,10 @@ void BonjourAccount::usernameNotInStream(BonjourContactConnection *conn)
 BonjourContact *BonjourAccount::verifyUser(BonjourContactConnection *conn, const QString &user)
 {
 	// First Check the User Exists
-	if (! contacts().keys().contains(user))
+	if (!contacts().value(user))
 		return NULL;
 
-	BonjourContact *c = (BonjourContact *) contacts()[user];
+	BonjourContact *c = (BonjourContact *) contacts().value(user);
 
 	if (c->getremoteAddress() != conn->getHostAddress())
 		return NULL;
