@@ -152,7 +152,7 @@ void JabberChatSession::slotUpdateDisplayName ()
 	XMPP::Jid jid = static_cast<JabberBaseContact*>(chatMembers.first())->rosterItem().jid();
 
 	if ( !mResource.isEmpty () )
-		jid.setResource ( mResource );
+		jid.withResource ( mResource );
 
 	QString statusText = i18nc("a contact's online status in parenthesis.", " (%1)",
 							  chatMembers.first()->onlineStatus().description() );
@@ -284,21 +284,16 @@ void JabberChatSession::sendNotification( Event event )
 
 	if( !members().isEmpty() && (send_state || send_msg_event) )
 	{
-		// create JID for us as sender
-		XMPP::Jid fromJid = static_cast<const JabberBaseContact*>(myself())->rosterItem().jid();
-		fromJid.setResource ( account()->resource () );
-
 		// create JID for the recipient
 		JabberContact *recipient = static_cast<JabberContact*>(members().first());
 		XMPP::Jid toJid = recipient->rosterItem().jid();
 
 		// set resource properly if it has been selected already
 		if ( !resource().isEmpty () )
-			toJid.setResource ( resource () );
+			toJid.withResource ( resource () );
 
 		XMPP::Message message;
 
-		message.setFrom ( fromJid );
 		message.setTo ( toJid );
 		if(send_msg_event)
 		{
@@ -352,14 +347,10 @@ void JabberChatSession::slotMessageSent ( Kopete::Message &message, Kopete::Chat
 		XMPP::Message jabberMessage;
 		JabberBaseContact *recipient = static_cast<JabberBaseContact*>(message.to().first());
 
-		XMPP::Jid jid = static_cast<const JabberBaseContact*>(message.from())->rosterItem().jid();
-		jid.setResource ( account()->configGroup()->readEntry( "Resource", QString() ) );
-		jabberMessage.setFrom ( jid );
-
 		XMPP::Jid toJid = recipient->rosterItem().jid();
 
 		if( !resource().isEmpty () )
-			toJid.setResource ( resource() );
+			toJid.withResource ( resource() );
 
 		jabberMessage.setTo ( toJid );
 
@@ -396,7 +387,8 @@ void JabberChatSession::slotMessageSent ( Kopete::Message &message, Kopete::Chat
 			if (message.format() ==  Qt::RichText)
 			{
 				JabberResource *bestResource = account()->resourcePool()->bestJabberResource(toJid);
-				if( bestResource && bestResource->features().canXHTML() )
+				if( bestResource && bestResource->features().test(
+						QStringList("http://jabber.org/protocol/xhtml-im")) )
 				{
 					QString xhtmlBody = message.escapedBody();
 
