@@ -21,7 +21,6 @@
 #include "jabberbasecontact.h"
 
 namespace Kopete { class MetaContact; }
-class JabberGroupChatManager;
 class JabberChatSession;
 
 class JabberGroupMemberContact : public JabberBaseContact
@@ -40,7 +39,8 @@ public:
 	 * Create custom context menu items for the contact
 	 * FIXME: implement manager version here?
 	 */
-	QPtrList<KAction> *customContextMenuActions ();
+	QList<KAction*> *customContextMenuActions ();
+	using JabberBaseContact::customContextMenuActions;
 
 	/**
 	 * Return message manager for this instance.
@@ -57,14 +57,14 @@ public slots:
 	/**
 	 * This is the JabberContact level slot for sending files.
 	 *
-	 * @param sourceURL The actual KURL of the file you are sending
+	 * @param sourceURL The actual KUrl of the file you are sending
 	 * @param fileName (Optional) An alternate name for the file - what the
 	 *                 receiver will see
 	 * @param fileSize (Optional) Size of the file being sent. Used when sending
 	 *                 a nondeterminate file size (such as over a socket)
 	 */
-	virtual void sendFile( const KURL &sourceURL = KURL(),
-		const QString &fileName = QString::null, uint fileSize = 0L );
+	virtual void sendFile( const KUrl &sourceURL = KUrl(),
+		const QString &fileName = QString(), uint fileSize = 0L );
 
 private slots:
 	/**
