@@ -258,11 +258,10 @@ void ICQAccount::connectWithPassword( const QString &password )
 
 		updateVersionUpdaterStamp();
 
-		Connection* c = setupConnection();
 		engine()->start( server, port, accountId(), password.left(8) );
 		engine()->setStatus( status, mInitialStatusMessage.message(), pres.xtrazStatus(),
 		                     mInitialStatusMessage.title(), pres.mood() );
-		engine()->connectToServer( c, server, port, true /* doAuth */ );
+		engine()->connectToServer( server, port );
 
 		mInitialStatusMessage = Kopete::StatusMessage();
 	}
@@ -521,7 +520,7 @@ void ICQAccount::addedInfoEventActionActivated( uint actionId )
 	case Kopete::AddedInfoEvent::InfoAction:
 		{
 			ICQUserInfoWidget* info = 0;
-			ICQContact *ct = dynamic_cast<ICQContact*>(contacts().value( event->contactId(), 0 ));
+			ICQContact *ct = dynamic_cast<ICQContact*>(contacts().value( event->contactId() ));
 			if ( ct )
 				info = new ICQUserInfoWidget( ct, Kopete::UI::Global::mainWidget() );
 			else
