@@ -27,6 +27,8 @@
 #include "contact.h"
 #include "kopete_export.h"
 
+class QDomNode;
+
 namespace Kopete
 {
 class Contact;
@@ -35,7 +37,7 @@ class FileTransferInfo;
 }
 
 class Client;
-class Connection;
+class ClientStream;
 class OscarContact;
 class OscarAccountPrivate;
 class QTextCodec;
@@ -126,11 +128,6 @@ public slots:
 	void slotGoOnline();
 
 protected:
-	/**
-	 * Setup a connection for a derived account
-	 */
-	Connection* setupConnection();
-
 	/**
 	 * Adds a contact to a meta contact
 	 */
@@ -185,6 +182,8 @@ protected slots:
 	void fileTransferRefused( const Kopete::FileTransferInfo& info );
 	void fileTransferAccept( Kopete::Transfer* t , const QString& fileName );
 
+	void chatroomRequest( ChatRoomHandler* handler );
+
 signals:
 
 	void accountDisconnected( Kopete::Account::DisconnectReason reason );
@@ -201,6 +200,8 @@ private:
 
 	QString sanitizedPlainMessage( const QString& message ) const;
 
+	QList<QDomNode> getElementsByTagNameCI( const QDomNode& node, const QString& tagName ) const;
+
 private slots:
 	/** Handler from socket errors from a connection */
 	void slotSocketError( int, const QString& );
@@ -215,6 +216,11 @@ private slots:
 	void slotIdentityPropertyChanged( Kopete::PropertyContainer *container, const QString &key,
 	                                  const QVariant &oldValue, const QVariant &newValue );
 
+	/**
+	 * Setup a ClientStream for engine
+	 */
+	void createClientStream( ClientStream **clientStream );
+
 private:
 	OscarAccountPrivate *d;
 
